package com.app.bloggerebookapp.activity;

import static com.app.bloggerebookapp.Config.BANNER_BOOK_DETAIL;
import static com.app.bloggerebookapp.Config.INTERSTITIAL_BOOK_DETAIL;

import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewStub;
import android.webkit.WebView;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.coordinatorlayout.widget.CoordinatorLayout;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.StaggeredGridLayoutManager;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import com.app.bloggerebookapp.Config;
import com.app.bloggerebookapp.R;
import com.app.bloggerebookapp.adapter.AdapterCategoryList;
import com.app.bloggerebookapp.adapter.AdapterRelated;
import com.app.bloggerebookapp.callback.CallbackPost;
import com.app.bloggerebookapp.database.prefs.AdsPref;
import com.app.bloggerebookapp.database.prefs.SharedPref;
import com.app.bloggerebookapp.database.room.AppDatabase;
import com.app.bloggerebookapp.database.room.DAO;
import com.app.bloggerebookapp.helper.AdsManager;
import com.app.bloggerebookapp.helper.Constant;
import com.app.bloggerebookapp.helper.Tools;
import com.app.bloggerebookapp.model.Post;
import com.app.bloggerebookapp.rest.RestAdapter;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.facebook.shimmer.ShimmerFrameLayout;
import com.google.android.material.snackbar.Snackbar;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import retrofit2.Call;
import retrofit2.Callback;
import retrofit2.Response;

public class ActivityBookDetail extends AppCompatActivity {

    public static final String TAG = "ActivityBookDetail";
    SharedPref sharedPref;
    Tools tools;
    Post post;
    String labels;
    ImageView bookImage;
    TextView bookTitle;
    String categories;
    RecyclerView recyclerViewCategory;
    TextView txtUncategorized;
    LinearLayout lytLabelUncategorized;
    WebView bookDescription;
    ImageButton btnBookmark, btnShare, btnSearch;
    FrameLayout customViewContainer;
    SwipeRefreshLayout swipeRefreshLayout;
    DAO db;
    Button btnReadBook;
    LinearLayout lytContent;
    ShimmerFrameLayout lytShimmer;
    CoordinatorLayout parentView;
    AdsManager adsManager;
    AdsPref adsPref;
    private boolean flag_read_later;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Tools.getTheme(this);
        setContentView(R.layout.activity_book_detail);

        db = AppDatabase.getDb(this).get();
        tools = new Tools(this);
        sharedPref = new SharedPref(this);
        adsManager = new AdsManager(this);
        adsPref = new AdsPref(this);

        adsManager.loadBannerAd(BANNER_BOOK_DETAIL);
        adsManager.loadInterstitialAd(INTERSTITIAL_BOOK_DETAIL, 1);

        LinearLayout nativeAdView = findViewById(R.id.native_ad_view);
        Tools.setNativeAdStyle(this, nativeAdView, Config.NATIVE_AD_STYLE_BOOK_DETAIL);
        adsManager.loadNativeAd(Config.NATIVE_AD_BOOK_DETAIL, Config.NATIVE_AD_STYLE_BOOK_DETAIL);

        Tools.setNavigation(this);

        post = (Post) getIntent().getSerializableExtra(Constant.EXTRA_OBJC);
        labels = getIntent().getStringExtra(Constant.EXTRA_LABELS);

        setupToolbar();
        initView();
        refreshBookmark();
        loadData();
        displayData();
        initShimmerLayout();
    }

    public void setupToolbar() {
        Toolbar toolbar = findViewById(R.id.toolbar);
        Tools.setupToolbar(this, toolbar, "", true);
    }

    private void initView() {
        parentView = findViewById(R.id.parent_view);
        lytContent = findViewById(R.id.lyt_content);
        lytShimmer = findViewById(R.id.shimmer_view_container);

        bookImage = findViewById(R.id.book_image);
        bookTitle = findViewById(R.id.book_title);
        bookDescription = findViewById(R.id.book_description);
        recyclerViewCategory = findViewById(R.id.recycler_view_category);
        txtUncategorized = findViewById(R.id.txt_label_uncategorized);
        lytLabelUncategorized = findViewById(R.id.lyt_label_uncategorized);
        btnShare = findViewById(R.id.btn_share);
        btnSearch = findViewById(R.id.btn_search);
        customViewContainer = findViewById(R.id.customViewContainer);

        swipeRefreshLayout = findViewById(R.id.swipe_refresh_layout);
        swipeRefreshLayout.setColorSchemeResources(R.color.color_light_primary);
        swipeRefreshLayout.setOnRefreshListener(this::refreshData);

        btnReadBook = findViewById(R.id.btn_read_book);
        btnReadBook.setOnClickListener(v -> {
            if (!tools.verifyPermissions()) {
                return;
            }

            Intent intent = new Intent(getApplicationContext(), ActivityPdfView.class);
            intent.putExtra(Constant.EXTRA_OBJC, post);
            intent.putExtra(Constant.EXTRA_LABELS, labels);
            startActivity(intent);
            sharedPref.savePostId(post.id);
            adsManager.destroyBannerAd();

            if (sharedPref.getInterstitialAdCounter() >= adsPref.getInterstitialAdInterval()) {
                sharedPref.updateInterstitialAdCounter(1);
                adsManager.showInterstitialAd();
            } else {
                sharedPref.updateInterstitialAdCounter(sharedPref.getInterstitialAdCounter() + 1);
            }
        });

        btnBookmark = findViewById(R.id.btn_bookmark);
        onBookmarkPage();

    }

    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == 1) {
            if (grantResults[0] == PackageManager.PERMISSION_GRANTED) {
                Intent intent = new Intent(getApplicationContext(), ActivityPdfView.class);
                intent.putExtra(Constant.EXTRA_OBJC, post);
                intent.putExtra(Constant.EXTRA_LABELS, labels);
                startActivity(intent);
                sharedPref.savePostId(post.id);
                adsManager.destroyBannerAd();
                sharedPref.updateInterstitialAdCounter(sharedPref.getInterstitialAdCounter() + 1);
            }
        }
    }

    private void onBookmarkPage() {
        btnBookmark.setOnClickListener(v -> {
            String str;
            if (flag_read_later) {
                db.deleteBookmark(post.id);
                str = getString(R.string.msg_bookmark_removed);
            } else {
                db.insertBookmark(
                        post.id,
                        post.title,
                        post.content,
                        labels,
                        0,
                        System.currentTimeMillis()
                );
                str = getString(R.string.msg_bookmark_added);
            }
            Snackbar.make(parentView, str, Snackbar.LENGTH_SHORT).show();
            refreshBookmark();
        });
    }

    private void refreshBookmark() {
        flag_read_later = db.getBookmark(post.id) != null;
        if (flag_read_later) {
            btnBookmark.setImageResource(R.drawable.ic_bookmark_white);
        } else {
            btnBookmark.setImageResource(R.drawable.ic_bookmark_outline_white);
        }
    }

    private void refreshData() {
        swipeProgress(true);
        new Handler().postDelayed(() -> swipeProgress(false), 1000);
    }

    private void displayData() {
        new Handler().postDelayed(() -> swipeProgress(false), 0);
    }

    private void loadData() {

        bookTitle.setText(post.title);

        if (post.labels.size() > 0) {
            categories = String.valueOf(post.labels).replace("[", "").replace("]", "").replace(", ", ",");
            List<String> arrayListCategories = Arrays.asList(categories.split(","));
            recyclerViewCategory.setLayoutManager(new LinearLayoutManager(this, LinearLayoutManager.HORIZONTAL, false));
            AdapterCategoryList adapter = new AdapterCategoryList(this, arrayListCategories);
            recyclerViewCategory.setAdapter(adapter);
            adapter.setOnItemClickListener((view, items, pos) -> {
                Intent intent = new Intent(getApplicationContext(), ActivityCategoryDetail.class);
                intent.putExtra(Constant.EXTRA_OBJC, items.get(pos));
                startActivity(intent);
                adsManager.destroyBannerAd();
            });
            if (sharedPref.showRelatedBook().equals("true")) {
                requestRelatedAPI(post.labels.get(0));
            }
        } else {
            categories = getString(R.string.txt_uncategorized);
            if (sharedPref.getParent().equals("bookmark")) {
                txtUncategorized.setText(labels);
                txtUncategorized.setOnClickListener(v -> {
                    Intent intent = new Intent(getApplicationContext(), ActivityCategoryDetail.class);
                    intent.putExtra(Constant.EXTRA_OBJC, labels);
                    startActivity(intent);
                    adsManager.destroyBannerAd();
                });
            } else {
                txtUncategorized.setText(categories);
            }
            txtUncategorized.setVisibility(View.VISIBLE);
            lytLabelUncategorized.setVisibility(View.VISIBLE);
            recyclerViewCategory.setVisibility(View.GONE);
        }

        Document htmlData = Jsoup.parse(post.content);
        Elements elements = htmlData.select("img");

        if (elements.hasAttr("src")) {
            Glide.with(this)
                    .load(elements.get(0).attr("src").replace(" ", "%20"))
                    .diskCacheStrategy(DiskCacheStrategy.ALL)
                    .placeholder(R.drawable.bg_button_transparent)
                    .into(bookImage);

            bookImage.setOnClickListener(v -> {
                Intent intent = new Intent(getApplicationContext(), ActivityImageDetail.class);
                intent.putExtra("name", post.title);
                intent.putExtra("image", elements.get(0).attr("src").replace(" ", "%20"));
                startActivity(intent);
            });
        }

        if (htmlData.select("img").first() != null) {
            Element element = htmlData.select("img").first();
            if (element != null && element.hasAttr("src")) {
                element.remove();
            }
        }

        if (htmlData.select("iframe").first() != null) {
            Element element = htmlData.select("iframe").first();
            if (element != null && element.hasAttr("src")) {
                element.remove();
            }
        }

        Tools.displayPostDescription(this, bookDescription, htmlData.toString());

        btnSearch.setOnClickListener(v -> {
            Intent intent = new Intent(getApplicationContext(), ActivitySearch.class);
            startActivity(intent);
            adsManager.destroyBannerAd();
        });

        btnShare.setOnClickListener(v -> Tools.shareContent(ActivityBookDetail.this, post.title));

    }

    private void requestRelatedAPI(String category) {
        TextView txtRelated = findViewById(R.id.txt_related);
        RelativeLayout lytRelated = findViewById(R.id.lyt_related);
        RecyclerView recyclerViewRelated = findViewById(R.id.recycler_view_related);
        recyclerViewRelated.setLayoutManager(new StaggeredGridLayoutManager(sharedPref.getBookColumns(), StaggeredGridLayoutManager.VERTICAL));
        recyclerViewRelated.setNestedScrollingEnabled(false);
        AdapterRelated adapterRelated = new AdapterRelated(this, recyclerViewRelated, new ArrayList<>());
        recyclerViewRelated.setAdapter(adapterRelated);
        recyclerViewRelated.setNestedScrollingEnabled(false);

        List<String> apiKeys = Arrays.asList(sharedPref.getAPIKey().replace(", ", ",").split(","));
        int totalKeys = (apiKeys.size() - 1);
        String apiKey;
        if (sharedPref.getApiKeyPosition() > totalKeys) {
            apiKey = apiKeys.get(0);
            sharedPref.updateApiKeyPosition(0);
        } else {
            apiKey = apiKeys.get(sharedPref.getApiKeyPosition());
        }

        Call<CallbackPost> callbackCallRelated = RestAdapter.createApiPosts(sharedPref.getBloggerId()).getRelatedPosts(category, Constant.DISPLAY_POST_ORDER, apiKey);
        callbackCallRelated.enqueue(new Callback<CallbackPost>() {
            public void onResponse(@NonNull Call<CallbackPost> call, @NonNull Response<CallbackPost> response) {
                CallbackPost resp = response.body();
                if (resp != null) {
                    txtRelated.setText(getString(R.string.txt_related));
                    new Handler(Looper.getMainLooper()).postDelayed(() -> {
                        lytRelated.setVisibility(View.VISIBLE);
                    }, 2000);
                    adapterRelated.insertData(resp.items);
                    adapterRelated.setOnItemClickListener((view, obj, position) -> {
                        Intent intent = new Intent(getApplicationContext(), ActivityBookDetail.class);
                        intent.putExtra(Constant.EXTRA_OBJC, obj);
                        startActivity(intent);
                        sharedPref.savePostId(obj.id);
                        adsManager.destroyBannerAd();
                    });
                    if (resp.items.size() == 1) {
                        txtRelated.setText("");
                        lytRelated.setVisibility(View.GONE);
                    }
                    sharedPref.updateRetryToken(0);
                } else {
                    if (sharedPref.getRetryToken() < Constant.MAX_RETRY_TOKEN) {
                        if (sharedPref.getApiKeyPosition() >= totalKeys) {
                            sharedPref.updateApiKeyPosition(0);
                        } else {
                            sharedPref.updateApiKeyPosition(sharedPref.getApiKeyPosition() + 1);
                        }
                        sharedPref.updateRetryToken(sharedPref.getRetryToken() + 1);
                    } else {
                        sharedPref.updateRetryToken(0);
                    }
                    lytRelated.setVisibility(View.GONE);
                }
            }

            public void onFailure(@NonNull Call<CallbackPost> call, @NonNull Throwable th) {
                Log.e("onFailure", "" + th.getMessage());
                if (!call.isCanceled()) {
                    lytRelated.setVisibility(View.GONE);
                }
            }
        });
    }

    private void swipeProgress(final boolean show) {
        if (!show) {
            swipeRefreshLayout.setRefreshing(false);
            lytContent.setVisibility(View.VISIBLE);
            lytShimmer.setVisibility(View.GONE);
            lytShimmer.stopShimmer();
            return;
        }
        swipeRefreshLayout.post(() -> {
            swipeRefreshLayout.setRefreshing(true);
            lytContent.setVisibility(View.GONE);
            lytShimmer.setVisibility(View.VISIBLE);
            lytShimmer.startShimmer();
        });
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem menuItem) {
        if (menuItem.getItemId() == android.R.id.home) {
            onBackPressed();
        } else {
            return super.onOptionsItemSelected(menuItem);
        }
        return true;
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
        adsManager.destroyBannerAd();
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        adsManager.destroyBannerAd();
    }

    @Override
    public void onResume() {
        super.onResume();
        refreshBookmark();
        adsManager.resumeBannerAd(BANNER_BOOK_DETAIL);
    }

    private void initShimmerLayout() {
        ViewStub stub = findViewById(R.id.lytShimmerView);
        stub.setLayoutResource(R.layout.shimmer_book_detail);
        stub.inflate();
    }

}
