package com.app.bloggerebookapp.activity;

import static com.app.bloggerebookapp.Config.BANNER_BOOK_DETAIL;

import android.annotation.SuppressLint;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Build;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.Looper;
import android.text.InputFilter;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.appcompat.app.AlertDialog;
import androidx.appcompat.app.AppCompatActivity;
import androidx.appcompat.widget.Toolbar;
import androidx.coordinatorlayout.widget.CoordinatorLayout;

import com.app.bloggerebookapp.R;
import com.app.bloggerebookapp.callback.CallbackPostDetail;
import com.app.bloggerebookapp.database.prefs.SharedPref;
import com.app.bloggerebookapp.database.room.AppDatabase;
import com.app.bloggerebookapp.database.room.DAO;
import com.app.bloggerebookapp.helper.AdsManager;
import com.app.bloggerebookapp.helper.Constant;
import com.app.bloggerebookapp.helper.InputFilterIntRange;
import com.app.bloggerebookapp.helper.Tools;
import com.app.bloggerebookapp.model.Post;
import com.facebook.shimmer.ShimmerFrameLayout;
import com.github.barteksc.pdfviewer.PDFView;
import com.github.barteksc.pdfviewer.link.DefaultLinkHandler;
import com.github.barteksc.pdfviewer.listener.OnLoadCompleteListener;
import com.github.barteksc.pdfviewer.listener.OnPageChangeListener;
import com.github.barteksc.pdfviewer.listener.OnPageErrorListener;
import com.github.barteksc.pdfviewer.scroll.DefaultScrollHandle;
import com.github.barteksc.pdfviewer.scroll.ScrollHandle;
import com.github.barteksc.pdfviewer.util.FitPolicy;
import com.google.android.material.appbar.AppBarLayout;
import com.google.android.material.dialog.MaterialAlertDialogBuilder;
import com.google.android.material.progressindicator.CircularProgressIndicator;
import com.google.android.material.snackbar.Snackbar;
import com.shockwave.pdfium.PdfDocument;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;

@SuppressWarnings("deprecation")
public class ActivityPdfViewNotif extends AppCompatActivity implements OnPageChangeListener, OnLoadCompleteListener, OnPageErrorListener {

    private static final String TAG = "loadPDF";
    private PDFView pdfView;
    Toolbar toolbar;
    TextView toolbarTitle;
    TextView toolbarSubTitle;
    boolean flag = true;
    AdsManager adsManager;
    InputStream inputStream = null;
    OutputStream outputStream = null;
    private ShimmerFrameLayout lytShimmer;
    CoordinatorLayout parentView;
    TextView txtPage;
    AppBarLayout lytTop;
    LinearLayout lytBottom;
    Tools tools;
    File folderPath;
    File pdfPath;
    String fileName;
    String fileExtension = "pdf";
    View lytFailed;
    Button btnRetry;
    ImageButton btnInfo;
    TextView txtPercentage;
    TextView txtLoading;
    CircularProgressIndicator progressBar;
    private final Handler handler = new Handler();
    SharedPref sharedPref;
    DAO db;
    ImageButton btnBookmark;
    int savedReadingPages = 0;
    int lastReadPage = 0;
    private boolean flag_read_later;
    CallbackPostDetail post;
    String labels;
    String pdfUrl;
    List<File> list = null;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Tools.getTheme(this);
        setContentView(R.layout.activity_pdf_view);
        db = AppDatabase.getDb(this).get();

        sharedPref = new SharedPref(this);
        tools = new Tools(this);
        Tools.setNavigation(this);

        post = (CallbackPostDetail) getIntent().getSerializableExtra(Constant.EXTRA_OBJC);
        labels = getIntent().getStringExtra(Constant.EXTRA_LABELS);

        if (db.getBookmark(post.id) != null) {
            Post page = db.getBookmark(post.id);
            savedReadingPages = page.page_position;
            Log.d(TAG, "last page visited : " + savedReadingPages);
        }

        adsManager = new AdsManager(this);
        adsManager.loadBannerAd(BANNER_BOOK_DETAIL);

        toolbar = findViewById(R.id.toolbar);
        toolbarTitle = findViewById(R.id.toolbar_title);
        toolbarSubTitle = findViewById(R.id.toolbar_subtitle);
        lytBottom = findViewById(R.id.lytBottom);
        lytTop = findViewById(R.id.appBarLayout);
        lytShimmer = findViewById(R.id.shimmerViewContainer);
        parentView = findViewById(R.id.coordinatorLayout);
        txtPage = findViewById(R.id.txtPage);
        lytFailed = findViewById(R.id.lytFailed);
        btnRetry = findViewById(R.id.failed_retry);
        txtPercentage = findViewById(R.id.txtPercentage);
        txtLoading = findViewById(R.id.txtLoading);
        progressBar = findViewById(R.id.progressBar);
        btnBookmark = findViewById(R.id.btnBookmark);

        btnInfo = findViewById(R.id.btnInfo);
        if (sharedPref.showBookDetail().equals("true")) {
            btnInfo.setVisibility(View.GONE);
        }

        pdfView = findViewById(R.id.pdfView);
        pdfView.setVisibility(View.GONE);
        pdfView.setOnClickListener(v -> {
            if (flag) {
                tools.fullScreenMode(lytTop, lytBottom, true);
                flag = false;
            } else {
                tools.fullScreenMode(lytTop, lytBottom, false);
                flag = true;
            }
        });

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
            folderPath = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOCUMENTS) + "/." + getString(R.string.app_name) + "/");
        } else {
            folderPath = new File(Environment.getExternalStorageDirectory() + "/." + getString(R.string.app_name) + "/");
        }

        if (sharedPref.allowSaveBook().equals("true")) {
            loadPdf();
        } else {
            loadFile(false);
        }

        refreshBookmark();
        Tools.setupToolbar(this, toolbar, "", true);
        setupToolbar();
    }

    private void loadPdf() {
        if (folderPath.exists()) {
            list = Arrays.asList(Objects.requireNonNull(folderPath.listFiles((dir, name) -> name.contains(Tools.reformatFileName(post.title)))));
            if (list.size() > 0) {
                int totalFiles = list.size();
                fileName = list.get(totalFiles - 1).toString();
                pdfPath = new File(fileName);
                loadPdfFromFile(pdfPath);
                Log.d(TAG, "pdf file found and try to load it");
            } else {
                loadFile(true);
                Log.d(TAG, "no pdf file found, load from server first");
            }
            Log.d(TAG, "folder exist");
        } else {
            loadFile(true);
            Log.d(TAG, "folder not exist");
        }
    }

    private void setupToolbar() {
        toolbarTitle.setText(post.title);
    }

    @Override
    public void onPageChanged(int page, int pageCount) {
        lastReadPage = page;
        txtPage.setText(String.format("%s %s / %s", "", page + 1, pageCount));
        toolbarSubTitle.setText(String.format("%s %s %s %s", "Page", page + 1, "of", pageCount));
        onBookmarkPage(page);
    }

    @Override
    public void loadComplete(int nbPages) {
        swipeProgress(false);
        toolbarSubTitle.setVisibility(View.VISIBLE);
        findViewById(R.id.btnJumpPage).setOnClickListener(v -> showPageDialog(pdfView.getPageCount()));
        findViewById(R.id.btnShare).setOnClickListener(v -> Tools.shareContent(ActivityPdfViewNotif.this, Tools.reformatFileName(post.title)));
        btnInfo.setOnClickListener(v -> bookInfo());
        PdfDocument.Meta meta = pdfView.getDocumentMeta();
        Log.d(TAG, "title = " + meta.getTitle());
        Log.d(TAG, "author = " + meta.getAuthor());
        Log.d(TAG, "subject = " + meta.getSubject());
        Log.d(TAG, "keywords = " + meta.getKeywords());
        Log.d(TAG, "creator = " + meta.getCreator());
        Log.d(TAG, "producer = " + meta.getProducer());
        Log.d(TAG, "creationDate = " + meta.getCreationDate());
        Log.d(TAG, "modDate = " + meta.getModDate());
        printBookmarksTree(pdfView.getTableOfContents(), "-");
    }

    public void printBookmarksTree(List<PdfDocument.Bookmark> tree, String sep) {
        for (PdfDocument.Bookmark b : tree) {
            Log.d(TAG, String.format("%s %s, p %d", sep, b.getTitle(), b.getPageIdx()));
            if (b.hasChildren()) {
                printBookmarksTree(b.getChildren(), sep + "-");
            }
        }
    }

    public void bookInfo() {
        Intent intent = new Intent(getApplicationContext(), ActivityBookDetail.class);
        intent.putExtra(Constant.EXTRA_OBJC, post);
        intent.putExtra(Constant.EXTRA_LABELS, labels);
        startActivity(intent);
        sharedPref.savePostId(post.id);
        adsManager.destroyBannerAd();
    }

    @Override
    public void onPageError(int page, Throwable t) {
        Log.e(TAG, "Cannot load page " + page);
        showFailedView();
    }

    @Override
    public void onBackPressed() {
        super.onBackPressed();
        updateLastPageRead();
        adsManager.destroyBannerAd();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem menuItem) {
        if (menuItem.getItemId() == android.R.id.home) {
            onBackPressed();
        } else {
            return super.onOptionsItemSelected(menuItem);
        }
        return true;
    }

    private void updateLastPageRead() {
        flag_read_later = db.getBookmark(post.id) != null;
        if (flag_read_later) {
            db.updateBookmark(post.id, lastReadPage);
            Log.d(TAG, "update last page bookmarked");
        }
    }

    private void onBookmarkPage(int pagePosition) {
        btnBookmark.setOnClickListener(v -> {
            String str;
            if (flag_read_later) {
                db.deleteBookmark(post.id);
                str = getString(R.string.msg_bookmark_removed);
            } else {
                db.insertBookmark(
                        post.id,
                        Tools.reformatFileName(post.title),
                        post.content,
                        labels,
                        pagePosition,
                        System.currentTimeMillis()
                );
                str = getString(R.string.msg_bookmark_added);
            }
            Snackbar.make(parentView, str, Snackbar.LENGTH_SHORT).show();
            refreshBookmark();
        });
    }

    public void showPageDialog(int totalPages) {
        final MaterialAlertDialogBuilder dialog = new MaterialAlertDialogBuilder(ActivityPdfViewNotif.this);
        LayoutInflater inflater = LayoutInflater.from(ActivityPdfViewNotif.this);
        View view = inflater.inflate(R.layout.dialog_jump_page, null);

        LinearLayout lytDialogHeader = view.findViewById(R.id.lytDialogHeader);
        LinearLayout lytDialogContent = view.findViewById(R.id.lytDialogContent);
        if (sharedPref.getIsDarkTheme()) {
            lytDialogHeader.setBackgroundColor(getResources().getColor(R.color.color_dark_toolbar));
            lytDialogContent.setBackgroundColor(getResources().getColor(R.color.color_dark_background));
        }

        TextView txtInputPageNumber = view.findViewById(R.id.txtInputPageNumber);
        txtInputPageNumber.setText(String.format("%s %s - %s", getString(R.string.input_page_number), "1", totalPages));

        EditText edtPageNumber = view.findViewById(R.id.edtPageNumber);
        edtPageNumber.setHint(String.format("%s - %s", "1", totalPages));

        edtPageNumber.requestFocus();
        tools.showKeyboard(true);

        InputFilterIntRange rangeFilter = new InputFilterIntRange(1, totalPages);
        edtPageNumber.setFilters(new InputFilter[]{rangeFilter});
        edtPageNumber.setOnFocusChangeListener(rangeFilter);

        dialog.setView(view);
        dialog.setCancelable(false);

        AlertDialog alertDialog = dialog.create();

        TextView btnPositive = view.findViewById(R.id.btnPositive);
        btnPositive.setOnClickListener(v -> new Handler().postDelayed(() -> {
            if (!edtPageNumber.getText().toString().equals("")) {
                int pageNumber = (Integer.parseInt(edtPageNumber.getText().toString()) - 1);
                new Handler().postDelayed(() -> pdfView.jumpTo(pageNumber, true), 200);
                tools.showKeyboard(false);
                alertDialog.dismiss();
            } else {
                Snackbar.make(parentView, getString(R.string.msg_input_page), Snackbar.LENGTH_SHORT).show();
            }
        }, 300));

        TextView btnNegative = view.findViewById(R.id.btnNegative);
        btnNegative.setOnClickListener(v -> new Handler().postDelayed(() -> {
            tools.showKeyboard(false);
            alertDialog.dismiss();
        }, 300));

        alertDialog.show();
    }

    private void swipeProgress(final boolean show) {
        if (show) {
            lytShimmer.setVisibility(View.VISIBLE);
            lytShimmer.startShimmer();
        } else {
            lytShimmer.setVisibility(View.GONE);
            txtPercentage.setVisibility(View.GONE);
            txtLoading.setVisibility(View.GONE);
            progressBar.setVisibility(View.GONE);
            lytShimmer.stopShimmer();
        }
    }

    private void showFailedView() {
        swipeProgress(false);
        pdfView.setVisibility(View.GONE);
        lytFailed.setVisibility(View.VISIBLE);
        ((TextView) findViewById(R.id.no_item_message)).setText(getString(R.string.msg_no_item));
        btnRetry.setOnClickListener(v -> {
            swipeProgress(true);
            lytFailed.setVisibility(View.GONE);
            pdfView.setVisibility(View.GONE);
            loadFile(true);
        });
    }

    public void loadFile(boolean saveToStorage) {

        Document htmlData = Jsoup.parse(post.content);
        Elements books = htmlData.select("iframe");
        if (books.hasAttr("src")) {
            String url = books.get(0).attr("src");
            if (url.contains("drive.google.com")) {
                pdfUrl = url.replace("https://", "").replace("http://", "");
                List<String> urls = Arrays.asList(pdfUrl.split("/"));
                downloadTask("https://drive.google.com/uc?export=download&id=" + urls.get(3), saveToStorage);
            } else {
                pdfUrl = url;
                downloadTask(pdfUrl, saveToStorage);
            }
        }

    }

    private void downloadTask(String pdfUrl, boolean saveToStorage) {
        DownloadFileFromURL downloadFileFromURL = new DownloadFileFromURL(saveToStorage);
        downloadFileFromURL.execute(pdfUrl);
    }

    @SuppressLint("StaticFieldLeak")
    public class DownloadFileFromURL extends AsyncTask<String, String, Void> {

        public boolean saveToStorage;

        public DownloadFileFromURL(boolean saveToStorage) {
            this.saveToStorage = saveToStorage;
        }

        public void onPreExecute() {
            txtPercentage.setText("0%");
            txtLoading.setText(getString(R.string.txt_preparing_book));
            super.onPreExecute();
        }

        public Void doInBackground(String... f_url) {
            try {
                URL url = new URL(f_url[0]);
                URLConnection connection = url.openConnection();
                connection.connect();
                int lengthOfFile = connection.getContentLength();
                inputStream = new BufferedInputStream(url.openStream(), 1024);
                if (saveToStorage) {
                    saveToStorage(lengthOfFile, inputStream, fileExtension);
                } else {
                    loadPdfFromInputStream(inputStream);
                }
            } catch (Exception e) {
                Log.e("Error: ", "error: " + e.getMessage());
                return null;
            }
            return null;
        }

        public void onProgressUpdate(String... progress) {

        }

        public void onPostExecute(Void file_url) {
            loadPdf();
        }
    }

    /** @noinspection IOStreamConstructor*/
    public void saveToStorage(int lengthOfFile, InputStream inputStream, String extension) {
        try {
            File dir;
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                dir = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOCUMENTS) + "/." + getString(R.string.app_name));
            } else {
                dir = new File(Environment.getExternalStorageDirectory() + "/." + getString(R.string.app_name));
            }
            boolean success = true;
            if (!dir.exists()) {
                success = dir.mkdirs();
            }
            if (success) {
                try {
                    Log.d(TAG, "File Size = " + lengthOfFile);
                    String fileName;
                    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.R) {
                        fileName = post.title + "_" + System.currentTimeMillis();
                    } else {
                        fileName = post.title;
                    }
                    outputStream = new FileOutputStream(dir + "/" + Tools.reformatFileName(fileName) + "." + extension);

                    byte[] data = new byte[4096];
                    int count;
                    int progress = 0;
                    while ((count = inputStream.read(data)) != -1) {
                        outputStream.write(data, 0, count);
                        progress += count;
                        int finalProgress = progress;
                        handler.post(() -> {
                            int percentage = ((finalProgress * 100) / lengthOfFile);
                            txtPercentage.setText(percentage + "%");
                            txtLoading.setText(getString(R.string.txt_loading_book));
                        });
                        Log.d(TAG, "Progress: " + progress + "/" + lengthOfFile + " >>>> " + (float) progress / lengthOfFile);
                    }
                    outputStream.flush();
                    loadPdf();
                    Log.d(TAG, "File saved successfully!");
                } catch (IOException e) {
                    e.printStackTrace();
                    loadFile(false);
                    Log.d(TAG, "Failed to save the file! " + e.getMessage());
                } finally {
                    if (inputStream != null) inputStream.close();
                    if (outputStream != null) outputStream.close();
                }
            } else {
                Log.d(TAG, "not success");
                loadFile(true);
            }
        } catch (IOException e) {
            e.printStackTrace();
            loadFile(true);
            Log.d(TAG, "Error to save the file! " + e.getMessage());
        }
    }

    public void loadPdfFromFile(File pdfPath) {

        new Handler(Looper.getMainLooper()).postDelayed(() -> {

            ScrollHandle scrollHandle;
            if (sharedPref.getScrollHandle().equals("true")) {
                scrollHandle = new DefaultScrollHandle(ActivityPdfViewNotif.this);
            } else {
                scrollHandle = null;
            }

            pdfView.fromFile(pdfPath)
                    .linkHandler(new DefaultLinkHandler(pdfView))
                    .defaultPage(savedReadingPages)
                    .onPageChange(ActivityPdfViewNotif.this)
                    .enableAnnotationRendering(true)
                    .onLoad(ActivityPdfViewNotif.this)
                    .scrollHandle(scrollHandle)
                    .spacing(0) // in dp
                    .onPageError(ActivityPdfViewNotif.this)
                    .swipeHorizontal(sharedPref.getSwipeHorizontal())
                    .pageSnap(true)
                    .autoSpacing(true)
                    .pageFling(true)
                    .pageFitPolicy(FitPolicy.WIDTH)
                    .onError(t -> {
                        loadFile(true);
                        Log.d(TAG, "failed load pdf and try reload from url " + t.getMessage());
                    })
                    .nightMode(sharedPref.getIsDarkTheme())
                    .load();
            pdfView.setVisibility(View.VISIBLE);
        }, Constant.DELAY_REFRESH);
    }

    public void loadPdfFromInputStream(InputStream inputStream) {

        new Handler(Looper.getMainLooper()).postDelayed(() -> {

            ScrollHandle scrollHandle;
            if (sharedPref.getScrollHandle().equals("true")) {
                scrollHandle = new DefaultScrollHandle(ActivityPdfViewNotif.this);
            } else {
                scrollHandle = null;
            }

            pdfView.fromStream(inputStream)
                    .linkHandler(new DefaultLinkHandler(pdfView))
                    .defaultPage(savedReadingPages)
                    .onPageChange(ActivityPdfViewNotif.this)
                    .enableAnnotationRendering(true)
                    .onLoad(ActivityPdfViewNotif.this)
                    .scrollHandle(scrollHandle)
                    .spacing(0) // in dp
                    .onPageError(ActivityPdfViewNotif.this)
                    .swipeHorizontal(sharedPref.getSwipeHorizontal())
                    .pageSnap(true)
                    .autoSpacing(true)
                    .pageFling(true)
                    .pageFitPolicy(FitPolicy.WIDTH)
                    .onError(t -> {
                        loadFile(false);
                        Log.d(TAG, "failed load pdf and try reload from url");
                    })
                    .nightMode(sharedPref.getIsDarkTheme())
                    .load();
            pdfView.setVisibility(View.VISIBLE);
        }, Constant.DELAY_REFRESH);
    }

    private void refreshBookmark() {
        flag_read_later = db.getBookmark(post.id) != null;
        if (flag_read_later) {
            btnBookmark.setImageResource(R.drawable.ic_bookmark_white);
        } else {
            btnBookmark.setImageResource(R.drawable.ic_bookmark_outline_white);
        }
    }

    @Override
    public void onResume() {
        super.onResume();
        refreshBookmark();
        adsManager.resumeBannerAd(BANNER_BOOK_DETAIL);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        lytShimmer.stopShimmer();
        adsManager.destroyBannerAd();
    }

}