package com.app.bloggerebookapp.adapter;

import static com.app.bloggerebookapp.Config.NATIVE_AD_BOOK_LIST;
import static com.solodroid.ads.sdk.util.Constant.ADMOB;
import static com.solodroid.ads.sdk.util.Constant.AD_STATUS_ON;
import static com.solodroid.ads.sdk.util.Constant.APPLOVIN;
import static com.solodroid.ads.sdk.util.Constant.APPLOVIN_MAX;
import static com.solodroid.ads.sdk.util.Constant.FACEBOOK;
import static com.solodroid.ads.sdk.util.Constant.FAN;
import static com.solodroid.ads.sdk.util.Constant.GOOGLE_AD_MANAGER;
import static com.solodroid.ads.sdk.util.Constant.STARTAPP;

import android.app.Activity;
import android.content.Context;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.ProgressBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.StaggeredGridLayoutManager;

import com.app.bloggerebookapp.Config;
import com.app.bloggerebookapp.R;
import com.app.bloggerebookapp.database.prefs.AdsPref;
import com.app.bloggerebookapp.database.prefs.SharedPref;
import com.app.bloggerebookapp.helper.AdsManager;
import com.app.bloggerebookapp.helper.Constant;
import com.app.bloggerebookapp.model.Post;
import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.bumptech.glide.request.RequestOptions;
import com.solodroid.ads.sdk.format.NativeAdViewHolder;

import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.select.Elements;

import java.util.List;

public class AdapterBook extends RecyclerView.Adapter<RecyclerView.ViewHolder> {

    private final int VIEW_PROG = 0;
    private final int VIEW_ITEM = 1;
    private final int VIEW_AD = 2;
    private final List<Post> items;
    private final Context context;
    private OnItemClickListener onItemClickListener;
    private boolean loading;
    private OnLoadMoreListener onLoadMoreListener;
    boolean scrolling = false;
    SharedPref sharedPref;
    AdsPref adsPref;
    AdsManager adsManager;

    public interface OnItemClickListener {
        void onItemClick(View view, Post obj, int position);
    }

    public void setOnItemClickListener(final OnItemClickListener mItemClickListener) {
        this.onItemClickListener = mItemClickListener;
    }

    public AdapterBook(Context context, RecyclerView view, List<Post> items) {
        this.items = items;
        this.context = context;
        this.sharedPref = new SharedPref(context);
        this.adsPref = new AdsPref(context);
        this.adsManager = new AdsManager((Activity) context);
        lastItemViewDetector(view);
        view.addOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrollStateChanged(@NonNull RecyclerView recyclerView, int newState) {
                if (newState == RecyclerView.SCROLL_STATE_DRAGGING) {
                    scrolling = true;
                } else if (newState == RecyclerView.SCROLL_STATE_IDLE) {
                    scrolling = false;
                }
                super.onScrollStateChanged(recyclerView, newState);
            }
        });
    }

    public static class OriginalViewHolder extends RecyclerView.ViewHolder {
        public TextView bookTitle;
        public TextView bookCategory;
        public ImageView bookImage;
        public LinearLayout lytParent;

        public OriginalViewHolder(View v) {
            super(v);
            bookTitle = v.findViewById(R.id.book_title);
            bookCategory = v.findViewById(R.id.category_name);
            bookImage = v.findViewById(R.id.book_image);
            lytParent = v.findViewById(R.id.lyt_parent);
        }
    }

    public static class ProgressViewHolder extends RecyclerView.ViewHolder {
        public ProgressBar progressBar;

        public ProgressViewHolder(View v) {
            super(v);
            progressBar = v.findViewById(R.id.progressBar);
        }
    }

    /** @noinspection ConstantValue*/
    @NonNull
    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder vh;
        if (viewType == VIEW_ITEM) {
            View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_book, parent, false);
            vh = new OriginalViewHolder(v);
        } else if (viewType == VIEW_AD) {
            View v;
            if (Config.NATIVE_AD_STYLE_BOOK_LIST.equals("small")) {
                v = LayoutInflater.from(parent.getContext()).inflate(com.solodroid.ads.sdk.R.layout.view_native_ad_radio, parent, false);
            } else if (Config.NATIVE_AD_STYLE_BOOK_LIST.equals("medium")) {
                v = LayoutInflater.from(parent.getContext()).inflate(com.solodroid.ads.sdk.R.layout.view_native_ad_news, parent, false);
            } else {
                v = LayoutInflater.from(parent.getContext()).inflate(com.solodroid.ads.sdk.R.layout.view_native_ad_medium, parent, false);
            }
            vh = new NativeAdViewHolder(v);
        } else {
            View v = LayoutInflater.from(parent.getContext()).inflate(R.layout.item_load_more, parent, false);
            vh = new ProgressViewHolder(v);
        }
        return vh;
    }

    @Override
    public void onBindViewHolder(@NonNull RecyclerView.ViewHolder holder, final int position) {
        if (holder instanceof OriginalViewHolder) {
            final Post p = items.get(position);
            final OriginalViewHolder vItem = (OriginalViewHolder) holder;

            vItem.bookTitle.setText(p.title);

            if (p.labels.size() > 0) {
                vItem.bookCategory.setText(p.labels.get(0));
            } else {
                vItem.bookCategory.setText(context.getString(R.string.txt_uncategorized));
            }

            Document htmlData = Jsoup.parse(p.content);
            Elements images = htmlData.select("img");
            if (images.hasAttr("src")) {
                Glide.with(context)
                        .load(images.get(0).attr("src").replace(" ", "%20"))
                        .apply(new RequestOptions().override(Constant.THUMBNAIL_WIDTH, Constant.THUMBNAIL_HEIGHT))
                        .diskCacheStrategy(DiskCacheStrategy.ALL)
                        .placeholder(R.drawable.bg_button_transparent)
                        .into(vItem.bookImage);
            }

            vItem.lytParent.setOnClickListener(view -> {
                if (onItemClickListener != null) {
                    onItemClickListener.onItemClick(view, p, position);
                }
            });

        } else if (holder instanceof NativeAdViewHolder) {

            final NativeAdViewHolder vItem = (NativeAdViewHolder) holder;

            vItem.loadNativeAd(context,
                    adsPref.getAdStatus(),
                    NATIVE_AD_BOOK_LIST,
                    adsPref.getAdType(),
                    adsPref.getBackupAds(),
                    adsPref.getAdMobNativeId(),
                    adsPref.getAdManagerNativeId(),
                    adsPref.getFanNativeUnitId(),
                    adsPref.getAppLovinNativeAdManualUnitId(),
                    adsPref.getAppLovinBannerMrecZoneId(),
                    sharedPref.getIsDarkTheme(),
                   false,
                    Config.NATIVE_AD_STYLE_BOOK_LIST,
                    android.R.color.transparent,
                    android.R.color.transparent
            );

            if (sharedPref.getIsDarkTheme()) {
                vItem.setNativeAdBackgroundResource(R.drawable.bg_native_dark);
            } else {
                vItem.setNativeAdBackgroundResource(R.drawable.bg_native_light);
            }

            vItem.setNativeAdPadding(
                    context.getResources().getDimensionPixelOffset(R.dimen.grid_space_books),
                    context.getResources().getDimensionPixelOffset(R.dimen.grid_space_books),
                    context.getResources().getDimensionPixelOffset(R.dimen.grid_space_books),
                    context.getResources().getDimensionPixelOffset(R.dimen.grid_space_books)
            );

            vItem.setNativeAdMargin(
                    context.getResources().getDimensionPixelOffset(R.dimen.grid_space_books),
                    context.getResources().getDimensionPixelOffset(R.dimen.grid_space_books),
                    context.getResources().getDimensionPixelOffset(R.dimen.grid_space_books),
                    context.getResources().getDimensionPixelOffset(R.dimen.grid_space_books)
            );

        } else {
            ((ProgressViewHolder) holder).progressBar.setIndeterminate(true);
        }

        if (getItemViewType(position) == VIEW_PROG || getItemViewType(position) == VIEW_AD) {
            StaggeredGridLayoutManager.LayoutParams layoutParams = (StaggeredGridLayoutManager.LayoutParams) holder.itemView.getLayoutParams();
            layoutParams.setFullSpan(true);
        } else {
            StaggeredGridLayoutManager.LayoutParams layoutParams = (StaggeredGridLayoutManager.LayoutParams) holder.itemView.getLayoutParams();
            layoutParams.setFullSpan(false);
        }

    }

    public void insertData(List<Post> items) {
        setLoaded();
        int positionStart = getItemCount();
        int itemCount = items.size();
        this.items.addAll(items);
        notifyItemRangeInserted(positionStart, itemCount);
    }

    public void insertDataWithNativeAd(List<Post> items) {
        setLoaded();
        int positionStart = getItemCount();

        if (adsPref.getAdStatus().equals(AD_STATUS_ON) && NATIVE_AD_BOOK_LIST != 0) {
            if (sharedPref.getBookColumns() == 3) {
                if (items.size() >= Constant.NATIVE_AD_INDEX_3_COLUMNS)
                    items.add(Constant.NATIVE_AD_INDEX_3_COLUMNS, new Post());
                Log.d("INSERT_DATA", "3 columns");
            } else {
                if (items.size() >= Constant.NATIVE_AD_INDEX_2_COLUMNS)
                    items.add(Constant.NATIVE_AD_INDEX_2_COLUMNS, new Post());
                Log.d("INSERT_DATA", "2 columns");
            }
        }

        int itemCount = items.size();
        this.items.addAll(items);
        notifyItemRangeInserted(positionStart, itemCount);
    }

    @SuppressWarnings("SuspiciousListRemoveInLoop")
    public void setLoaded() {
        loading = false;
        for (int i = 0; i < getItemCount(); i++) {
            if (items.get(i) == null) {
                items.remove(i);
                notifyItemRemoved(i);
            }
        }
    }

    public void setLoading() {
        if (getItemCount() != 0) {
            this.items.add(null);
            notifyItemInserted(getItemCount() - 1);
            loading = true;
        }
    }

    public void resetListData() {
        this.items.clear();
        notifyDataSetChanged();
    }

    @Override
    public int getItemCount() {
        return items.size();
    }

    @Override
    public int getItemViewType(int position) {
        Post post = items.get(position);
        if (post != null) {
            if (post.title == null || post.title.equals("")) {
                return VIEW_AD;
            }
            return VIEW_ITEM;
        } else {
            return VIEW_PROG;
        }
    }

    public void setOnLoadMoreListener(OnLoadMoreListener onLoadMoreListener) {
        this.onLoadMoreListener = onLoadMoreListener;
    }

    private void lastItemViewDetector(RecyclerView recyclerView) {
        if (recyclerView.getLayoutManager() instanceof StaggeredGridLayoutManager) {
            final StaggeredGridLayoutManager layoutManager = (StaggeredGridLayoutManager) recyclerView.getLayoutManager();
            recyclerView.addOnScrollListener(new RecyclerView.OnScrollListener() {
                @Override
                public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
                    super.onScrolled(recyclerView, dx, dy);
                    int lastPos = getLastVisibleItem(layoutManager.findLastVisibleItemPositions(null));
                    if (!loading && lastPos == getItemCount() - 1 && onLoadMoreListener != null) {
                        if (sharedPref.getBookColumns() == 3) {
                            if (adsPref.getAdStatus().equals(AD_STATUS_ON) && NATIVE_AD_BOOK_LIST != 0) {
                                switch (adsPref.getAdType()) {
                                    case ADMOB:
                                    case GOOGLE_AD_MANAGER:
                                    case FAN:
                                    case FACEBOOK:
                                    case STARTAPP:
                                    case APPLOVIN:
                                    case APPLOVIN_MAX: {
                                        int current_page = getItemCount() / (Constant.LOAD_MORE_3_COLUMNS + 1); //posts per page plus 1 Ad
                                        onLoadMoreListener.onLoadMore(current_page);
                                        break;
                                    }
                                    default: {
                                        int current_page = getItemCount() / (Constant.LOAD_MORE_3_COLUMNS);
                                        onLoadMoreListener.onLoadMore(current_page);
                                        break;
                                    }
                                }
                            } else {
                                int current_page = getItemCount() / (Constant.LOAD_MORE_3_COLUMNS);
                                onLoadMoreListener.onLoadMore(current_page);
                            }
                        } else {
                            if (adsPref.getAdStatus().equals(AD_STATUS_ON) && NATIVE_AD_BOOK_LIST != 0) {
                                switch (adsPref.getAdType()) {
                                    case ADMOB:
                                    case GOOGLE_AD_MANAGER:
                                    case FAN:
                                    case FACEBOOK:
                                    case STARTAPP:
                                    case APPLOVIN:
                                    case APPLOVIN_MAX: {
                                        int current_page = getItemCount() / (Constant.LOAD_MORE_2_COLUMNS + 1); //posts per page plus 1 Ad
                                        onLoadMoreListener.onLoadMore(current_page);
                                        break;
                                    }
                                    default: {
                                        int current_page = getItemCount() / (Constant.LOAD_MORE_2_COLUMNS);
                                        onLoadMoreListener.onLoadMore(current_page);
                                        break;
                                    }
                                }
                            } else {
                                int current_page = getItemCount() / (Constant.LOAD_MORE_2_COLUMNS);
                                onLoadMoreListener.onLoadMore(current_page);
                            }
                        }
                        loading = true;
                    }
                }
            });
        }
    }

    public interface OnLoadMoreListener {
        void onLoadMore(int current_page);
    }

    private int getLastVisibleItem(int[] into) {
        int lastIdx = into[0];
        for (int i : into) {
            if (lastIdx < i) lastIdx = i;
        }
        return lastIdx;
    }

}