package com.app.bloggerebookapp.fragment;

import android.content.Intent;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.RecyclerView;
import androidx.recyclerview.widget.StaggeredGridLayoutManager;

import com.app.bloggerebookapp.R;
import com.app.bloggerebookapp.activity.ActivityBookDetail;
import com.app.bloggerebookapp.activity.ActivityPdfView;
import com.app.bloggerebookapp.activity.MainActivity;
import com.app.bloggerebookapp.adapter.AdapterFavorite;
import com.app.bloggerebookapp.database.prefs.SharedPref;
import com.app.bloggerebookapp.database.room.AppDatabase;
import com.app.bloggerebookapp.database.room.DAO;
import com.app.bloggerebookapp.helper.Constant;
import com.app.bloggerebookapp.helper.Tools;
import com.app.bloggerebookapp.model.Post;

import java.util.ArrayList;
import java.util.List;

public class FragmentBookmark extends Fragment {

    private View rootView;
    LinearLayout lytNoFavorite;
    RecyclerView recyclerView;
    private AdapterFavorite adapterFavorite;
    SharedPref sharedPref;
    Tools tools;
    private DAO db;

    @Nullable
    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        rootView = inflater.inflate(R.layout.fragment_bookmark, container, false);
        db = AppDatabase.getDb(getContext()).get();
        if (getActivity() != null) {
            sharedPref = new SharedPref(getActivity());
            tools = new Tools(getActivity());
        }
        recyclerView = rootView.findViewById(R.id.recyclerView);
        lytNoFavorite = rootView.findViewById(R.id.lyt_no_favorite);
        recyclerView.setLayoutManager(new StaggeredGridLayoutManager(sharedPref.getBookColumns(), StaggeredGridLayoutManager.VERTICAL));

        //set data and list adapter
        adapterFavorite = new AdapterFavorite(getActivity(), recyclerView, new ArrayList<>());
        recyclerView.setAdapter(adapterFavorite);

        // on item list clicked
        adapterFavorite.setOnItemClickListener((v, obj, position) -> {

            if (!sharedPref.showBookDetail().equals("true")) {
                if (!tools.verifyPermissions()) {
                    return;
                }
            }

            Intent intent;
            if (sharedPref.showBookDetail().equals("true")) {
                intent = new Intent(getActivity(), ActivityBookDetail.class);
            } else {
                intent = new Intent(getActivity(), ActivityPdfView.class);
            }
            intent.putExtra(Constant.EXTRA_OBJC, obj);
            intent.putExtra(Constant.EXTRA_LABELS, obj.categories);
            startActivity(intent);

            sharedPref.savePostId(obj.id);
            sharedPref.setParent("bookmark");

            ((MainActivity) getActivity()).showInterstitialAd();
            ((MainActivity) getActivity()).destroyBannerAd();

        });

        return rootView;
    }

    @Override
    public void onResume() {
        super.onResume();
        displayData(db.getAllBookmark());
    }

    private void displayData(final List<Post> posts) {
        adapterFavorite.resetListData();
        List<Post> items = db.getAllBookmark();
        adapterFavorite.insertData(items);
        showNoItemView(false);
        if (posts.size() == 0) {
            showNoItemView(true);
        }
    }

    private void showNoItemView(boolean show) {
        ((TextView) rootView.findViewById(R.id.no_item_message)).setText(R.string.no_bookmark_found);
        if (show) {
            lytNoFavorite.setVisibility(View.VISIBLE);
        } else {
            lytNoFavorite.setVisibility(View.GONE);
        }
    }

}
